@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2000-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2000-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -493,7 +493,7 @@ accept_opts(L, S, FamilyOpts) ->
     case
         getopts(
           L,
-          [active, nodelay, keepalive, delay_send, priority, linger]
+          [active, nodelay, keepalive, delay_send, priority, linger, reuseaddr]
           ++ FamilyOpts)
     of
 	{ok, Opts} ->
@@ -564,31 +564,56 @@ peeloff(S, AssocId) ->
 %%
 send(S, Data, OptList) when is_port(S), is_list(OptList) ->
     ?DBG_FORMAT("prim_inet:send(~p, _, ~p)~n", [S,OptList]),
-    Mref = monitor(port, S),
-    MrefBin = term_to_binary(Mref, [local]),
-    MrefBinSize = byte_size(MrefBin),
-    MrefBinSize = MrefBinSize band 16#FFFF,
+    send(S, Data, OptList, monitor(port, S), make_ref()).
+
+send(S, Data, OptList, Mref, Sref) ->
+    SrefBin = term_to_binary(Sref, [local]),
+    SrefBinSize = byte_size(SrefBin),
+    SrefBinSize = SrefBinSize band 16#FFFF,
     try
         erlang:port_command(
-          S, [<<MrefBinSize:16,MrefBin/binary>>, Data], OptList)
+          S, [<<SrefBinSize:16,SrefBin/binary>>, Data], OptList)
     of
         false -> % Port busy when nosuspend option was passed
 	    ?DBG_FORMAT("prim_inet:send() -> {error,busy}~n", []),
             {error,busy};
         true ->
             receive
-                {inet_reply,S,Status,Mref} ->
+                {inet_reply,S,Sref} ->
+                    %% This causes a wait even though nosuspend was used.
+                    %% It only happens when the OS send operation returns
+                    %% that it would block, which should only happen
+                    %% for SCTP (seqpacket), never for UDP (dgram)
+                    %%
+                    %% To fix this we probably need to pass down
+                    %% the nosuspend option to inform inet_drv
+                    %% to not use driver_select and send a late second reply.
+                    %%
+                    ?DBG_FORMAT(
+                       "prim_inet:send(~p,,,) Waiting~n",
+                       [S]),
+                    receive
+                        {inet_reply,S,ok,Sref} ->
+                            send(S, Data, OptList, Mref, make_ref());
+                        {'DOWN',Mref,_,_,_Reason} ->
+                            ?DBG_FORMAT(
+                               "prim_inet:send(~p,,,) 'DOWN' ~p~n",
+                               [S,_Reason]),
+                            {error,closed}
+                    end;
+                {inet_reply,S,Status,Sref} ->
                     demonitor(Mref, [flush]),
                     Status;
                 {'DOWN',Mref,_,_,_Reason} ->
                     ?DBG_FORMAT(
-                       "prim_inet:send_recv_reply(~p, _) 'DOWN' ~p~n",
+                       "prim_inet:send(~p,,,) 'DOWN' ~p~n",
                        [S,_Reason]),
                     {error,closed}
             end
     catch error: _ ->
 	    ?DBG_FORMAT("prim_inet:send() -> {error,einval}~n", []),
-	     {error,einval}
+            demonitor(Mref, [flush]),
+            {error,einval}
     end.
 
 send(S, Data) ->
@@ -1552,6 +1577,7 @@ enc_opt(show_econnreset) -> ?INET_LOPT_TCP_SHOW_ECONNRESET;
 enc_opt(line_delimiter)  -> ?INET_LOPT_LINE_DELIM;
 enc_opt(raw)             -> ?INET_OPT_RAW;
 enc_opt(bind_to_device)  -> ?INET_OPT_BIND_TO_DEVICE;
+enc_opt(read_ahead)      -> ?INET_LOPT_TCP_READ_AHEAD;
 enc_opt(debug)           -> ?INET_OPT_DEBUG;
 % Names of SCTP opts:
 enc_opt(sctp_rtoinfo)	 	   -> ?SCTP_OPT_RTOINFO;
@@ -1623,6 +1649,7 @@ dec_opt(?INET_LOPT_TCP_SHOW_ECONNRESET) -> show_econnreset;
 dec_opt(?INET_LOPT_LINE_DELIM)      -> line_delimiter;
 dec_opt(?INET_OPT_RAW)              -> raw;
 dec_opt(?INET_OPT_BIND_TO_DEVICE) -> bind_to_device;
+dec_opt(?INET_LOPT_TCP_READ_AHEAD) -> read_ahead;
 dec_opt(?INET_OPT_DEBUG)          -> debug;
 dec_opt(I) when is_integer(I)     -> undefined.
 
@@ -1735,6 +1762,7 @@ type_opt_1(read_packets)    -> uint;
 type_opt_1(netns)           -> binary;
 type_opt_1(show_econnreset) -> bool;
 type_opt_1(bind_to_device)  -> binary;
+type_opt_1(read_ahead)      -> bool;
 type_opt_1(debug)           -> bool;
 %% 
 %% SCTP options (to be set). If the type is a record type, the corresponding
